@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Ximian Inc.
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
@@ -48,6 +48,7 @@ static CamelFolderInfo *vee_get_folder_info(CamelStore *store, const char *top,
 static void camel_vee_store_class_init (CamelVeeStoreClass *klass);
 static void camel_vee_store_init       (CamelVeeStore *obj);
 static void camel_vee_store_finalise   (CamelObject *obj);
+static void construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, CamelException *ex);
 
 static CamelStoreClass *camel_vee_store_parent;
 
@@ -82,8 +83,9 @@ camel_vee_store_class_init (CamelVeeStoreClass *klass)
 	store_class->delete_folder = vee_delete_folder;
 	store_class->get_folder_info = vee_get_folder_info;
 	store_class->free_folder_info = camel_store_free_folder_info_full;
-
-	store_class->sync = vee_sync;
+	((CamelServiceClass *)store_class)->construct = construct;
+	
+	/* store_class->sync = vee_sync; */
 	store_class->get_trash = vee_get_trash;
 	store_class->get_junk = vee_get_junk;
 }
@@ -95,13 +97,25 @@ camel_vee_store_init (CamelVeeStore *obj)
 
 	/* we dont want a vtrash/vjunk on this one */
 	store->flags &= ~(CAMEL_STORE_VTRASH | CAMEL_STORE_VJUNK);	
+}
 
+static void
+construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, CamelException *ex)
+{
+	/*  CamelStore *store = (CamelStore *)service;
+	 CamelVeeStore *obj = (CamelVeeStore *)service; */
+	 
+	 ((CamelServiceClass *) camel_vee_store_parent)->construct(service, session, provider, url, ex);
+	 
 	/* Set up unmatched folder */
+#ifdef VEE_UNMATCHED_ENABLE
 	obj->unmatched_uids = g_hash_table_new (g_str_hash, g_str_equal);
 	obj->folder_unmatched = (CamelVeeFolder *)camel_object_new (camel_vee_folder_get_type ());
 	camel_vee_folder_construct (obj->folder_unmatched, store, CAMEL_UNMATCHED_NAME, _("Unmatched"), CAMEL_STORE_FOLDER_PRIVATE);
+	camel_db_create_vfolder (store->cdb, _("Unmatched"), NULL);
+#endif
+	 
 }
-
 static void
 cvs_free_unmatched(void *key, void *value, void *data)
 {
@@ -123,7 +137,7 @@ camel_vee_store_finalise (CamelObject *obj)
  *
  * Create a new #CamelVeeStore object.
  * 
- * Returns new #CamelVeeStore object
+ * Returns: new #CamelVeeStore object
  **/
 CamelVeeStore *
 camel_vee_store_new (void)
@@ -179,7 +193,7 @@ vee_get_folder (CamelStore *store, const char *folder_name, guint32 flags, Camel
 	char *name, *p;
 
 	vf = (CamelVeeFolder *)camel_vee_folder_new(store, folder_name, flags);
-	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0) {
+	if (vf && ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0)) {
 		/* Check that parents exist, if not, create dummy ones */
 		name = alloca(strlen(((CamelFolder *)vf)->full_name)+1);
 		strcpy(name, ((CamelFolder *)vf)->full_name);
@@ -333,7 +347,8 @@ vee_get_folder_info(CamelStore *store, const char *top, guint32 flags, CamelExce
 	g_hash_table_destroy(infos_hash);
 
 	/* and always add UNMATCHED, if scanning from top/etc */
-	if (top == NULL || top[0] == 0 || strncmp(top, CAMEL_UNMATCHED_NAME, strlen(CAMEL_UNMATCHED_NAME)) == 0) {
+	#warning "comment it out well"
+	if (0 && (top == NULL || top[0] == 0 || strncmp(top, CAMEL_UNMATCHED_NAME, strlen(CAMEL_UNMATCHED_NAME)) == 0)) {
 		info = camel_folder_info_new ();
 		url = camel_url_new("vfolder:", NULL);
 		camel_url_set_path(url, ((CamelService *)store)->url->path);
